! trackwind, Cécile Agosta, July 2022
! track changes in winds (momentum budget)
!
module trackwind
    implicit none
    ! track_wind : activation of trackwind
    logical, parameter :: track_wind = .false.
    ! ntrackwind : number of routines changing winds tracked
    integer, parameter :: ntrackwind = 6 ! ntrackwind : number of tracked routines
    ! momentum : horizontal wind
    ! ==========================
    ! delta_u, delta_v : cumulative change in wind components
    real, allocatable, save :: delta_u(:, :, :, :) ! (mx, my, mz, ntrackwind)
    real, allocatable, save :: delta_v(:, :, :, :) ! (mx, my, mz, ntrackwind)
    ! uairDY_save, vairDY_save : Save state before routine call
    real, allocatable :: uairDY_save(:, :, :)
    real, allocatable :: vairDY_save(:, :, :)
    ! define tracked routines
    integer itw ! iteration for tracked wind chances
    character(len = 6), parameter :: name_wind(ntrackwind) = (/ 'dgz', 'fil', 'cor', 'dif', 'tur', 'lbc' /)
    ! index in the array
    integer, parameter :: i_dyndgz = 1 ! dyndgz : Horizontal Pressure Gradient Force
    integer, parameter :: i_dynfil = 2 ! dynfil : Horizontal filtering
    integer, parameter :: i_coriol = 3 ! coriol : Coriolis
    integer, parameter :: i_turhor = 4 ! turhor : Horizontal diffusion
    integer, parameter :: i_turabl = 5 ! turabl : Boundary Layer Turbulence
    integer, parameter :: i_lbcnud = 6 ! lbcnud : Lateral Boundary Condition
    ! for out_nc outputs
    real, allocatable, save :: delta_u_NCsave(:, :, :, :) ! (mx, my, mz, ntrackwind)
    real, allocatable, save :: delta_v_NCsave(:, :, :, :) ! (mx, my, mz, ntrackwind)
    ! ddelta_var = delta_var - delta_var_save
    real, allocatable :: ddelta_var(:, :, :)
    ! for outice outputs
    real, allocatable, save :: duIB(:, :, :, :) ! duIB : delta_u in m s-1 h-1
    real, allocatable, save :: dvIB(:, :, :, :) ! dvIB : delta_v in m s-1 h-1
    real, allocatable, save :: delta_u_IBsave(:, :, :, :) ! delta_u_IBsave : delta_u from previous time save
    real, allocatable, save :: delta_v_IBsave(:, :, :, :) ! delta_v_IBsave : delta_v from previous time save
    ! track dyndgz
    ! track_dgz : activation of trackwind in dyndgz
    logical, parameter :: track_dgz = .false.
    ! ntrackdgz : number of contribution to dgz tracked
    integer, parameter :: ntrackdgz = 2 ! ntrackdgz : number of tracked contribution in dyndgz
    ! delta_u_dgz, delta_v_dgz : cumulative change in wind components
    real, allocatable, save :: delta_u_dgz(:, :, :, :)
    real, allocatable, save :: delta_v_dgz(:, :, :, :)
    character(len = 6), parameter :: name_dgz(ntrackdgz) = (/ 'adv', 'pgf' /)
    ! index in the array
    integer, parameter :: i_dgzadv = 1 ! dgzadv : Advection contribution to Horizontal PGF
    integer, parameter :: i_dgzpgf = 2 ! dgzpgf : Horizontal PGF
    ! for out_nc outputs
    real, allocatable, save :: delta_u_dgz_NCsave(:, :, :, :)
    real, allocatable, save :: delta_v_dgz_NCsave(:, :, :, :)
    ! for outice outputs
    real, allocatable, save :: dudgzIB(:, :, :, :) ! dudgzIB : delta_u_dgz in m s-1 h-1
    real, allocatable, save :: dvdgzIB(:, :, :, :) ! dvdgzIB : delta_v_dgz in m s-1 h-1
    real, allocatable, save :: delta_u_dgz_IBsave(:, :, :, :) ! delta_u_dgz_IBsave : delta_u_dgz from previous time save
    real, allocatable, save :: delta_v_dgz_IBsave(:, :, :, :) ! delta_v_dgz_IBsave : delta_v_dgz from previous time save
    ! temporary variables
    real :: dudt_t
    real, allocatable :: dudt_tm1(:, :, :, :)
    real :: c1a_t(ntrackdgz)
    real, allocatable :: c1a_tm1(:, :, :, :)
    real, allocatable :: c1a_tm2(:, :, :, :)
    
    ! ! momentum : vertical wind psig
    ! ! =============================
    ! real :: psigDY_save(mx, my, mz) ! Save state before routine call
    ! ! dyndps : Mass Continuity
    ! real, save :: dp_dyndps(mx, my, mz)
    ! ! dynfil : Filtering
    ! real, save :: dp_dynfil(mx, my, mz)
    ! ! water : qv
    ! real :: qvDY_save(mx, my, mz) ! Save state before routine call
    ! real, save :: dq_dynadv(mx, my, mz) ! Advection
    ! real, save :: dq_turhor(mx, my, mz) ! Horizontal diffusion
    ! real, save :: dq_turabl(mx, my, mz) ! Turbulence
    ! real, save :: dq_sspray(mx, my, mz) ! Sea spray
    ! real, save :: dq_hydgen(mx, my, mz) ! Microphysics
    ! real, save :: dq_lbcnud(mx, my, mz) ! Lateral boundaries
    ! real, save :: dq_cvagen(mx, my, mz) ! Convection

contains
    
    subroutine trackwind_init()
        use mardim, only : mx, my, mz
        use mar_ib, only : ml
        implicit none
        ! allocates
        allocate(uairDY_save(mx, my, mz))
        allocate(vairDY_save(mx, my, mz))
        allocate(delta_u(mx, my, mz, ntrackwind))
        allocate(delta_v(mx, my, mz, ntrackwind))
        allocate(delta_u_NCsave(mx, my, mz, ntrackwind))
        allocate(delta_v_NCsave(mx, my, mz, ntrackwind))
        allocate(ddelta_var(mx, my, mz))
        allocate(duIB(mx, my, ml, ntrackwind))
        allocate(dvIB(mx, my, ml, ntrackwind))
        allocate(delta_u_IBsave(mx, my, mz, ntrackwind))
        allocate(delta_v_IBsave(mx, my, mz, ntrackwind))
        ! initial values
        uairDY_save = 0.
        vairDY_save = 0.
        delta_u = 0.
        delta_v = 0.
        delta_u_NCsave = 0.
        delta_v_NCsave = 0.
        ddelta_var = 0.
        duIB = 0.
        dvIB = 0.
        delta_u_IBsave = 0.
        delta_v_IBsave = 0.
    end subroutine trackwind_init
    
    subroutine trackdgz_init()
        use mardim, only : mx, my, mz
        use mar_ib, only : ml
        implicit none
        ! allocates
        allocate(delta_u_dgz(mx, my, mz, ntrackdgz))
        allocate(delta_v_dgz(mx, my, mz, ntrackdgz))
        allocate(delta_u_dgz_NCsave(mx, my, mz, ntrackdgz))
        allocate(delta_v_dgz_NCsave(mx, my, mz, ntrackdgz))
        allocate(dudgzIB(mx, my, ml, ntrackdgz))
        allocate(dvdgzIB(mx, my, ml, ntrackdgz))
        allocate(delta_u_dgz_IBsave(mx, my, mz, ntrackdgz))
        allocate(delta_v_dgz_IBsave(mx, my, mz, ntrackdgz))
        allocate(dudt_tm1(mx, my, mz, ntrackdgz))
        allocate(c1a_tm1(mx, my, mz, ntrackdgz))
        allocate(c1a_tm2(mx, my, mz, ntrackdgz))
        ! initial values
        delta_u_dgz = 0.
        delta_v_dgz = 0.
        delta_u_dgz_NCsave = 0.
        delta_v_dgz_NCsave = 0.
        dudgzIB = 0.
        dvdgzIB = 0.
        delta_u_dgz_IBsave = 0.
        delta_v_dgz_IBsave = 0.
        dudt_t = 0.
        dudt_tm1 = 0.
        c1a_t = 0.
        c1a_tm1 = 0.
        c1a_tm2 = 0.
    end subroutine trackdgz_init

end module trackwind
